#ifndef SLUB_H
#define SLUB_H

#include "types.h"
#include "memlayout.h"
#include "riscv.h"
#include "spinlock.h"

enum { PAGE_FREE, PAGE_BUDDY, PAGE_SLUB, PAGE_RESERVE };

extern char end[];
#define PA2PGREF_ID(p) (((p)-KERNBASE)/PGSIZE)
#define PGREF_MAX_ENTRIES PA2PGREF_ID(PHYSTOP)
#define ALIGN_SIZE(size, aligns) (((size - 1) / aligns + 1) * aligns)
#define KMEM_PAGE_NUM 4 //分给KMEM_CACHE结构体序列四页的自由空间，最多可以创建数百个kmem_cache
#define PAGE_NUM_PER_SLUB(size) \
  (size < PGSIZE ? 4 : ((size / PGSIZE) + 1) * 4) //每个slab包含多少页，如果对象很大那么一个块也要跟着大一点

#define SLUB_NUM 9
struct cache_area {
  void *base;
  uint32 size;
  void *freelist;
};

/*出于简化，我们不考虑CPU和node区分的两种情况，只使用一条partial list。因此
也不会有per cpu的partial list*/
struct kmem_cache { 
  /* kmem_cache_cpu */
  void **freelist;   /* Pointer to next available object */
  uint64 tid; /* Globally unique transaction id */
  struct page *page; /* The slab from which we are allocating */

  /* Used for retrieving partial slabs, etc. */
  int refcount;
  uint64 min_partial;
  uint32 size;         /* The size of an object including metadata */
  uint32 object_size;  /* The size of an object without metadata */
  uint32 offset; /* Free pointer offset */
  uint64 nr_page_per_slub;

  void (*ctor)(void *);
  uint32 inuse;        /* Offset to metadata */
  uint32 align;        /* Alignment */
  uint32 red_left_pad; /* Left redzone padding size */
  const char *name;          /* Name (only for display!) */
  uint64 nr_partial;
  struct page *partial_list;     /* partial List of slabs */
};




//     _____________________
//     |    |    |    |    |      要义是在连续的，长度以页为单位的不定长内存区间内实现一个表格结构
//     |  s |  s |  s |  s |      但我们只能靠page结构体实现这一点。
//     |____|____|____|____|      换句话说连续的几页是个group，我们把表与表的link与表内的freelist结构都保存在header
//        s   s+1  s+2  s+3
//     这是一个slab，里面包含4页，我们在每一页都记住开头的位置，那么就能根据地址找到从属的slab
//

struct page {
    //COW:
    int refcount;
    //SLUB ALLOCATOR:
    uint32 active_obj_count; //未回收的对象数量
    uint64 flags;
    void *freelist; //由header管理,其它的页不设置该字段,其指向本slab页面内部维护的freelist的第一个对象节点
    struct kmem_cache *slab;
    struct page *header; //the beginning page of a slab
    struct page *next;  //指向下一个slab(的header页)，该字段只由header管理
};

extern const uint32 kmem_cache_objsize[];
extern const char *kmem_cache_name[];
extern struct kmem_cache *slub_allocator[SLUB_NUM];
extern struct page pages[PGREF_MAX_ENTRIES];
extern struct spinlock pages_lock;
extern struct cache_area cache_region;
extern uint32 cache_tid;

// 函数声明
void page_init(void);
void *init_object_list(void *addr, uint32 objsize, uint32 size);
void kmem_cache_init(void);
struct kmem_cache *alloc_kmem_cache(void);
// void inline free_kmem_cache(struct kmem_cache *cache);
void *cache_create(const char *name, uint32 size, uint32 aligns, int flags, void *func(void *));
void *cache_alloc_pages(struct kmem_cache *cache);
struct kmem_cache *kmem_cache_create(const char *name, uint32 size, uint32 aligns, int flags, void *func(void *));
// int kmem_cache_destroy(struct kmem_cache *cache);
void *kmem_cache_alloc(struct kmem_cache *cache);
void kmem_cache_free(void *obj);
void slub_allocator_init(void);
void *kmalloc_(uint32 size);
void kfree_(const void *addr);
void stress_test();
void simple_allocation_test();
















#endif